#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2023-03-26 02:20:28 (ywatanabe)"

from .scripts.main import main

__copyright__ = "Copyright (C) 2023 Yusuke Watanabe"
__version__ = "0.11"
__license__ = "GPL3.0"
__author__ = "ywatanabe1989"
__author_email__ = "ywata1989@gmail.com"
__url__ = "https://github.com/ywatanabe1989/vChatGPT"

